﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Input;
using Syncfusion.SfSchedule.XForms;
using Xamarin.Forms;

namespace FSM.Views.Behavior
{
    public class SfScheduleBehavoir : Behavior<SfSchedule>
    {
        public SfSchedule AssociatedObject { get; private set; }

        public ICommand SetDateRangeCommand
        {
            get { return (ICommand) GetValue(SetDateRangeCommandProperty); }
            set { SetValue(SetDateRangeCommandProperty, value); }
        }
        
        public object LockObject { get; set; } = new object();

        #region Protected methods

        protected override void OnAttachedTo(SfSchedule bindable)
        {
            base.OnAttachedTo(bindable);
            AssociatedObject = bindable;
            bindable.BindingContextChanged += OnBindingContextChanged;
            bindable.VisibleDatesChangedEvent += VisibleDatesChanged;
        }

        

        protected override void OnBindingContextChanged()
        {
            base.OnBindingContextChanged();
            BindingContext = AssociatedObject.BindingContext;
        }

        protected override void OnDetachingFrom(SfSchedule bindable)
        {
            base.OnDetachingFrom(bindable);
            bindable.BindingContextChanged -= OnBindingContextChanged;
            bindable.VisibleDatesChangedEvent -= VisibleDatesChanged;
            AssociatedObject = null;
        }

        #endregion

        #region Private methods

        

        private void OnBindingContextChanged(object sender, EventArgs e)
        {
            OnBindingContextChanged();
        }

        

        private void VisibleDatesChanged(object sender, VisibleDatesChangedEventArgs args)
        {
            lock (LockObject)
            {
                if (SetDateRangeCommand == null)
                    return;
                var startDate = args.visibleDates?[0] ?? DateTime.MinValue;
                var endDate = args.visibleDates?[args.visibleDates.Count - 1] ?? DateTime.MinValue;

                var parameter = new Tuple<DateTime, DateTime>(startDate, endDate);

                if (_startDate.Date != startDate.Date || _endDate.Date != endDate.Date)
                {
                    _startDate = startDate;
                    _endDate = endDate;
                    if (SetDateRangeCommand.CanExecute(parameter))
                        SetDateRangeCommand.Execute(parameter);
                }
            }
        }

        #endregion

        #region Fields

        public static readonly BindableProperty SetDateRangeCommandProperty = BindableProperty.Create("SetDateRangeCommand",
            typeof(ICommand),
            typeof(SfScheduleBehavoir),
            null);


      
        private DateTime _startDate;
        private DateTime _endDate;

        #endregion
    }

    public class FSMScheduleAppointment: ScheduleAppointment
    {
    }
}